////////////////////////////////////////////////////////////////////
//
// File: 		procinfo.cpp
// Project: 	ntproc
//
// Desc: 
//
// Revisions: 	Created 09/2698
//
// Copyright(C) 1998, Tomas Restrepo. All rights reserved
//
///////////////////////////////////////////////////////////////////

// Includes
#define STRICT
#define  _WIN32_WINNT  0x0400
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "resource.h"
#include "psapi.h"
#include "ntproc/ntproc.h"

/*--
* HANDLE_DLGMSG macro
* allows handling dialog messages
--*/
#define HANDLE_DLGMSG(hwnd, msg, fn)								\
	case (msg): return SetDlgMsgResult (hwnd, msg,					\
		HANDLE_##msg((hwnd), (wParam), (lParam), (fn)))


/*--
* Globals
--*/
DWORD	pProcessIdList[1024];	// process id list (only up to 1024 processes)

//////////////////////////////////////////////////////////////////////
// Message Handlers for Main Dialog

/*--
* LB_SELCHANGE
*
* Here we get display information for a given process in the
* process list
*
* hwnd: Main Dialog hwnd, 
* iProcIndex: index in the process list
--*/
void ProcessList_OnSelChange (HWND hwnd, int iProcIndex)
{
	HANDLE		hProcess;
	TCHAR		string[15];

	if ((iProcIndex < 0) || (iProcIndex >= 50))
		return;

	hProcess = OpenProcess (PROCESS_QUERY_INFORMATION, FALSE, pProcessIdList[iProcIndex]);
	if (hProcess == NULL)
	{
		MessageBox (hwnd, _T("Cannot get Process info."), _T("ProcInfo"), 
			MB_OK | MB_ICONERROR);
		return;
	}

	wsprintf (string, _T("%d"), GetProcessHandleCount(hProcess));
	SendDlgItemMessage (hwnd, IDC_HANDLECOUNT, WM_SETTEXT, 0, (LPARAM)string);
	
	// clean up
	CloseHandle (hProcess);
} // ProcessList_OnSelChange()



/*--
* WM_INITDIALOG handler
--*/
BOOL MainDlg_OnInitDialog (HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	// TODO: fill the listbox with the process list
	HWND	hListBox = NULL;
	DWORD	dwSize = 50*sizeof(DWORD);
	HANDLE	hProcess = NULL;
	TCHAR	string[255];
	int		i;

	hListBox = GetDlgItem (hwnd, IDC_PROCESSLIST);
	memset (pProcessIdList, -1, dwSize); 

	if (EnumProcesses (pProcessIdList, dwSize, &dwSize) == FALSE)
	{
		// failed
		MessageBox (NULL, _T("Failed to get process list"), _T("ProcInfo"), 
				MB_OK | MB_ICONERROR);
		return FALSE;
	}

	// now show the process list on the list box
	for (i = 0; (i < 50),(pProcessIdList[i] != -1); i++)
	{
		wsprintf (string, _T("[%d]"), pProcessIdList[i]);
		ListBox_AddString (hListBox, string);
	}

	return TRUE;
} // MainDlg_OnInitDialog


/*--
* WM_DESTROY handler
--*/
void MainDlg_OnDestroy (HWND hwnd)
{
	// TODO: Clean up

} // MainDlg_OnDestroy


/*--
* WM_COMMAND handler
--*/
void MainDlg_OnCommand (HWND hwnd, int id, HWND hwndControl, UINT codeNotify)
{
	switch (id)
	{
	case IDOK:
		// TODO: display process info
		break;

	case IDCANCEL:
		// exit
		EndDialog (hwnd, 0);
		break;
	case IDC_PROCESSLIST:
		if (codeNotify == LBN_SELCHANGE)
		{
			int sel = SendMessage (hwndControl, LB_GETCURSEL, 0, 0);
			ProcessList_OnSelChange (hwnd, sel);
		}
		break;
	}
} // MainDlg_OnCommand



//////////////////////////////////////////////////////////////////////
// Main Procedures (callbacks, entry point, etc)

/*--
* MainDlgProc()
*
* Main dialog procedure. It's simple, since we're using Message
* crackers, so just make sure the procedures get the correct msg.
--*/
BOOL CALLBACK MainDlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		HANDLE_DLGMSG (hwnd, WM_INITDIALOG,	MainDlg_OnInitDialog);
		HANDLE_DLGMSG (hwnd, WM_DESTROY,	MainDlg_OnDestroy);
		HANDLE_DLGMSG (hwnd, WM_COMMAND,	MainDlg_OnCommand);
	}
	return FALSE;
	
} // MainDlgProc




/*--
* _tWinMain()
*
* Begin program execution. We simply create a Dialog using DialogBox
--*/
int WINAPI _tWinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
					  LPTSTR lpszCmdLine, int nCmdShow)
{

	DialogBox (hInstance, MAKEINTRESOURCE (IDD_MAINWINDOW), NULL, MainDlgProc);
	return 0;

} // _tWinMain






